<?php
class plgHikashoppaymentAtossips extends hikashopPaymentPlugin {

	var $multiple = true;
	var $name = 'atossips';
	var $uniq_merchant = true;

	//Acquirer response codes :
	var $reponse_codes = array (
		'00'=>"Transaction approved or processed successfully",
		'02'=>"Contact card issuer",
		'03'=>"Invalid acceptor",
		'04'=>"Retain the card",
		'05'=>"Do not honour",
		'07'=>"Retain the card, special circumstances",
		'08'=>"Approve after obtaining identification",
		'12'=>"Invalid transaction",
		'13'=>"Invalid amount",
		'14'=>"Invalid cardholder number",
		'15'=>"Card issuer unknown",
		'30'=>"Format error",
		'31'=>"Identifier of acquirer entity unknown",
		'33'=>"Card is past expiry date",
		'34'=>"Suspicion of fraud",
		'41'=>"Card lost",
		'43'=>"Card stolen",
		'51'=>"Insufficient funds or credit limit exceeded",
		'54'=>"Card is past expiry date",
		'56'=>"Card missing from file",
		'57'=>"Transaction not permitted for this cardholder",
		'58'=>"Transaction prohibited at terminal",
		'59'=>"Suspicion of fraud",
		'60'=>"The acceptor of the card must contact the Acquirer",
		'61'=>"Exceeds the withdrawal amount limit",
		'63'=>"Security rules not observed",
		'68'=>"Response not received or received too late",
		'90'=>"Momentary system crash",
		'91'=>"Card issuer inaccessible",
		'96'=>"System functioning incorrectly",
		'97'=>"Expiry of the global monitoring delay",
		'98'=>"Server unavailable network routing further request",
		'99'=>"Incident field initiator"
	);
	var $bank_request = array(
		'default_test' => 'https://payment-webinit.simu.sips-atos.com/paymentInit',
		'default_prod' => 'https://payment-webinit.sips-atos.com/paymentInit',
		'bnp_test' => 'https://payment-webinit-mercanet.test.sips-atos.com/paymentInit',
		'bnp_prod' => 'https://payment-webinit.mercanet.bnpparibas.net/paymentInit'
	);

	//Currencies code :
	var $sync_currencies = array(
		'EUR'=>'978','USD'=>'840','GBP'=>'826','JPY'=>'392','CAD'=>'124','AUD'=>'036','CHF'=>'756',
		'MXN'=>'484','TRY'=>'949','NZD'=>'554','NOK'=>'578','BRL'=>'986','ARS'=>'032','KHR'=>'116',
		'TWD'=>'901','SEK'=>'752','DKK'=>'208','KRW'=>'410','SGD'=>'702','XAF'=>'952'
	);

	var $accepted_currencies = array(
		'EUR','USD','GBP','JPY','CAD','AUD','CHF',
		'MXN','TRY','NZD','NOK','BRL','ARS','KHR',
		'TWD','SEK','DKK','KRW','SGD','XAF'
	);


	//backend spec config
	var $pluginConfig = array(
		'merchantID' => array("MERCHANT_ID",'input'),
		'secretKey'=>array("Secret Key",'input'),
		'keyVersion'=>array('Key Version','input'),
		'notification'=>array("ALLOW_NOTIFICATIONS_FROM_X",'boolean','0'),
		'bank' =>array('Bank', 'list', array(
			'default' => 'Default',
			'bnp' => 'Bnp Paribas (Mercanet)',
			'socg' => 'Société Générale'
		)),
		'testmode'=>array('TEST_MODE', 'boolean','0'),
		'debug' => array('DEBUG', 'boolean','0'),
		'return_url' => array('RETURN_URL', 'input'),
		'invalid_status' => array('INVALID_STATUS', 'orderstatus'),
		'pending_status' => array('PENDING_STATUS', 'orderstatus'),
		'verified_status' => array('VERIFIED_STATUS', 'orderstatus'),
	);

	/**
	 * Collect data & process with the checkout :
	 */
	function onAfterOrderConfirm(&$order,&$methods,$method_id){
		parent::onAfterOrderConfirm($order, $methods, $method_id);

		//Backend data check :
		if (empty ($this->payment_params->merchantID) )
		{
			$this->app->enqueueMessage('You have to configure a merchant ID for the Atos sips plugin payment first : check your plugin\'s parameters, on your website backend','error');
			return false;
		}

		if (empty($this->payment_params->keyVersion))
		{
			$this->app->enqueueMessage('You have to configure the Key Version for the Atos sips plugin payment first : check your plugin\'s parameters, on your website backend','error');
			return false;
		}
		if (empty($this->payment_params->secretKey))
		{
			$this->app->enqueueMessage('You have to configure the Secret Key for the Atos sips plugin payment first : check your plugin\'s parameters, on your website backend','error');
			return false;
		}

		//Set the notify url :
		$PostUrl = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=checkout&task=notify&notif_payment='.$this->name.'&tmpl=component&lang='.$this->locale . $this->url_itemid;
		$userPostUrl = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=checkout&task=notify&notif_payment='.$this->name.'&tmpl=component&user_return=1&lang='.$this->url_itemid;

		// Manage Transaction reference from bank standards :
		if ($this->payment_params->bank == 'socg') $transactionRef = '';
		else $transactionRef = $order->order_id;

		// Be sure that bank payment params is set :
		if (empty($this->payment_params->bank) || ($this->payment_params->bank == 'socg')) {
			$this->payment_params->bank = 'default';
		}

		// Define required url :
		$bank = $this->payment_params->bank;
		$environnement = ($this->payment_params->testmode == 0)? 'prod': 'test';

		$url = @$this->bank_request[$bank.'_'.$environnement];

		// Set required data for prepare the data string :
		$vars0 = array(
			"currencyCode" => @$this->sync_currencies[$this->currency->currency_code],
			"merchantId" => trim($this->payment_params->merchantID),
			"normalReturnUrl" => $userPostUrl,
			"amount" => str_replace(array('.',','),'',round($order->cart->full_total->prices[0]->price_value_with_tax,2)*100),
			"s10TransactionReference" => array("s10TransactionId" => $this->compute_transactionId()),
			"keyVersion" => trim ($this->payment_params->keyVersion),
			"automaticResponseUrl" => $PostUrl,
			"orderId" => $order->order_id,
			"statementReference" => $order->order_number //add the order number in the merchant bank account:
		);

		//Set to platform url
		$this->payment_params->url = $url;

		$data = $this->flatten_to_sips_payload($vars0);

		//Set Secret Key :
		$secretKey = utf8_encode($this->payment_params->secretKey);

		// built seal with payment request, hash :
		$seal = hash('sha256', utf8_encode ($data.$secretKey) );

		//Set requiried data for Atos sips :
		$vars = array (
			"Data" => $data,
			"InterfaceVersion" => "HP_2.3",
			"Seal" => $seal,
		);

		if($this->payment_params->debug)
		{
			$this->writeToLog("Data sent to Atos Sips: \n\n\n");
			$this->writeToLog(print_r($vars,true));
			$this->writeToLog("payment server url : \n\n\n");
			$this->writeToLog($url,true);
		}

		$this->vars = $vars;
		return $this->showPage('end');

	}

	/**
	 * Treatment start with Atos sips response /POST:
	 */
	function onPaymentNotification(&$statuses){

		//Set response from Atos Sips :
		parse_str(strtr($_POST["Data"], '=|', '=&'), $arr);
		$response = array_map('trim', $arr);

		//Load the parameters of the plugin thanks to order id/transaction reference :
		$order_id = (int)$response["orderId"];
		$dbOrder = $this->getOrder($order_id);
		$this->loadPaymentParams($dbOrder);

		//Debug check:
		if($this->payment_params->debug) {

			$this->writeToLog("Data recieved from Atos Sips :\n\n ");
			$this->writeToLog($_POST["Data"]);
		}

		//Error/Unknow Order id :
		if(empty($this->payment_params) ) {

			if($this->payment_params->debug)
			{
				$this->writeToLog("Unable to read Atos Sips Response or Unknow Order! \n\n");
			}
			return false;
		}
		//load order object
		$this->loadOrderData($dbOrder);

		//check notify url source : Payment platform/User return
		$user_return = !empty($_GET['user_return']);

		//Order reach the verified status -> redirect the user to the thank you page :
		if($user_return && $dbOrder->order_status == $this->payment_params->verified_status) {
			$EndUrl = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=checkout&task=after_end&order_id=' . $order_id . $this->url_itemid;
			$this->app->redirect($EndUrl);
		}

		//Get the secret from payment params ::
		$secretKey = utf8_encode($this->payment_params->secretKey);

		// built seal with payment request, hash :
		$seal = hash('sha256', utf8_encode ($_POST["Data"].$secretKey) );

		//Compare the recieve seal with the create one :
		if ($seal != $_POST["Seal"]) {

			if($this->payment_params->debug)
			{
				$this->writeToLog("Seals mismatch !!! Data has been modified! \n See the generated hash: \n\n");
				$this->writeToLog($seal,true);
				$this->writeToLog("compare with the recieved one :\n");
				$this->writeToLog($_POST["Seal"],true);
			}
			return('Invalid Seal!');
		}

		//Set the amount from our DB:
		$amount = str_replace(array('.',','),'',round($dbOrder->order_full_price,2,2)*100);

		if ($amount != $response["amount"]) {

			if($this->payment_params->debug)
			{
				$this->writeToLog("Amount mismatch !!!\n Waited amount: " . $amount . ",\n");
				$this->writeToLog("Amount recieved: " . $response["amount"] . "\n");
			}

			return('Invalid Amount!');
		}

		//Seals & Amount match! Continu process :

		//Determine order status with data from atos sips:
		$repCode = trim ($response["acquirerResponseCode"]);
		$notified = 0;

		switch( $repCode ) {

			//----------------------------------------------------------------------------------------------
			//Success :
			case '00':

				//Set mail for merchant:
				$details =  @$this->reponse_codes[$repCode];
				$details = "Response from Atos Sips " . $details . "\n\r /Atos Sips authorisation Id :".$response["authorisationId"];

				//Set status email for merchant:
				$status = "Accepted";

				//Set message for customer:
				$message ="";

				//Set order status:
				$order_status = $this->payment_params->verified_status;

				//Customer redirection transaction approved :
				$EndUrl = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=checkout&task=after_end&order_id='.$dbOrder->order_id . $this->url_itemid;

				//Notified the customer :
				$notified = 1;

				break;

			//----------------------------------------------------------------------------------------------
			//pending status :
			case '08':

				//Set mail for merchant:
				$details =  @$this->reponse_codes[$repCode];
				$details = "Response from Atos Sips " . $details . " : Pending status.\n\r /Atos Sips authorisation Id :".$response["authorisationId"];

				//Set status email for merchant:
				$status = "Pending";

				//Set message for customer:
				$message ="";

				//Set order status:
				$order_status = $this->payment_params->pending_status;

				//Customer redirection transaction pending :
				$EndUrl = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=checkout&task=after_end&order_id='.$dbOrder->order_id . $this->url_itemid;

				break;

			//----------------------------------------------------------------------------------------------
			//rejected :
			case '02':
			case '03':
			case '04':
			case '05':
			case '07':
			case '12':
			case '13':
			case '14':
			case '15':
			case '30':
			case '31':
			case '33':
			case '34':
			case '41':
			case '43':
			case '51':
			case '54':
			case '56':
			case '57':
			case '58':
			case '59':
			case '60':
			case '61':
			case '63':
			case '68':
			case '90':
			case '91':
			case '96':
			case '97':
			case '98':
			case '99':

				//Set message for merchant:
				$details =  @$this->reponse_codes[$repCode];
				$details = "Response from Atos Sips " . $details . " : Invalid status.\n\r /Atos Sips authorisation Id :".$response["authorisationId"];

				//Set status email for merchant:
				$status = "Declined";

				//Set message for customer:
				$message =JText::_("TRANSACTION_DECLINED");

				//Set order status:
				$order_status = $this->payment_params->invalid_status;

				//Customer redirection transaction refused :
				$EndUrl = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=order&task=cancel_order&order_id='.$dbOrder->order_id . $this->url_itemid;

				break;

			//---------------------------------------------------------------------------------------------
			default:

				//Set mail for merchant:
				$details = "Unknow response from Atos Sips " . $repCode."\n\r /Atos Sips authorisation Id :".$response["authorisationId"];

				//Set status email for merchant:
				$status = "Declined";

				//Set message for customer:
				$message =JText::_("TRANSACTION_DECLINED");

				//Set order status:
				$order_status = $this->payment_params->invalid_status;

				//Customer redirection transaction refused/status unknow/error :
				$EndUrl = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=order&task=cancel_order&order_id='.$dbOrder->order_id . $this->url_itemid;

				break;
			//----------------------------------------------------------------------------------------------
		}

		if($order_status == $dbOrder->order_status) {
			$this->writeToLog('status already there !');
			if($user_return) {
				$this->app->enqueueMessage($message);
				$this->app->redirect($EndUrl);
			}
			return;
		}

		//Prepare email for merchant:
		$url = HIKASHOP_LIVE.'administrator/index.php?option=com_hikashop&ctrl=order&task=edit&order_id='.$order_id;
		$order_text = "\r\n".JText::sprintf('NOTIFICATION_OF_ORDER_ON_WEBSITE',$dbOrder->order_number,HIKASHOP_LIVE);
		$order_text .= "\r\n".str_replace('<br/>',"\r\n",JText::sprintf('ACCESS_ORDER_WITH_LINK',$url));

		$email = new stdClass();
		$email->subject = JText::sprintf('PAYMENT_NOTIFICATION_FOR_ORDER','Atos Sips',$status,$dbOrder->order_number);
		$body = str_replace('<br/>',"\r\n",JText::sprintf('PAYMENT_NOTIFICATION_STATUS','Atos Sips',$status)).' '.JText::_('STATUS_NOT_CHANGED')
		."\r\n" . $details . "\r\n".$order_text;
		$email->body = $body;

		//history get order data:
		$history = new stdClass();
		$history->notified = $notified;
		$history->amount = $amount.$this->currency->currency_code;
		$history->data = $details;

		//Debug check:
		if($this->payment_params->debug) {

			$this->writeToLog("Transaction Result :\n ".$details."\n\n");
		}

		//Set status
		if (empty($user_return) ) {

			$this->modifyOrder($order_id, $order_status, $history, $email);
		}


		if($user_return) {
			$this->writeToLog();

			$this->app->enqueueMessage($message);
			$this->app->redirect($EndUrl);
		}
		return false;
	}

	//Set default backend option
	function getPaymentDefaultValues(&$element) {
		$element->payment_name='ATOS SIPS';
		$element->payment_description='You can pay by credit card using this payment method';
		$element->payment_images='MasterCard,VISA,Credit_card,American_Express';
		$element->payment_params->notification= 1;
		$element->payment_params->testmode= 0;
		$element->payment_params->debug = 0;
		$element->payment_params->invalid_status='cancelled';
		$element->payment_params->pending_status='created';
		$element->payment_params->verified_status='confirmed';
	}

	function compute_transactionId() {
		// this distributes an Id from 000000 to 999999 along the day, based on typical distribution of transaction during the day.
		$currentDateHour = date('H');
		$currentDateMin = date('i');
		$currentDateSec = date('s');
		$currentDateMillArr = explode(' ', microtime());
		$currentDateMill = (int)round($currentDateMillArr[0] * 1000);
		$intPeriodeMs = 0;
		$intPeriodeMs += $currentDateHour * 60 * 60 * 1000;
		$intPeriodeMs += $currentDateMin * 60 * 1000;
		$intPeriodeMs += $currentDateSec * 1000;
		$intPeriodeMs += $currentDateMill;

		$dblProjectionTid = 0;
		$dblProjectionTid += (-5.636E-26 * pow($intPeriodeMs, 4));
		$dblProjectionTid += (7.061E-18 * pow($intPeriodeMs, 3));
		$dblProjectionTid += (-6.692E-11 * pow($intPeriodeMs, 2));
		$dblProjectionTid += (8.566E-4 * pow($intPeriodeMs, 1));
		$dblProjectionTid = floor($dblProjectionTid);
		$intProjectionTid = (int) $dblProjectionTid;

		if ($intProjectionTid > 999999)
			$intProjectionTid = 999999;
		if ($intProjectionTid < 0)
			$intProjectionTid = 0;

		// Padding
		$strProjectionTid = $intProjectionTid;
		$strProjectionTid=str_pad($strProjectionTid, 6, "0", STR_PAD_LEFT);

		return $strProjectionTid;
	}

	// Paypage Post interface
	function flatten_to_sips_payload($input) {
		$keyStack = array();
		return implode("|", $this->flatten_undefined($input, $keyStack));
	}

	// utility method called by flatten_to_sips_payload and flatten_array
	// returns a single dimensional array that can be imploded as a string with the
	// required separator
	function flatten_undefined($object, $keyStack) {
		$result = array();
		if(is_array($object)){
			$result = array_merge($result, $this->flatten_array($object, $keyStack));
		}else if(!empty($keyStack)){
			$result[] = implode('.', $keyStack) . '=' . $object;
		}else{
			$result[] = $object;
		}
		return $result;
	}

	// utility method called by flatten_undefined or by itself
	// returns a single dimensional array representing this array
	function flatten_array($array, $keyStack) {
		$simpleValues = array();$result = array();

		foreach($array as $key => $value){
			if(is_int($key)){
				// Values without keys are added to results after ones having keys
				if(is_array($value)){
					$noKeyStack = array();
					$simpleValues = array_merge($simpleValues, $this->flatten_array($value, $noKeyStack));
				}else{
					$simpleValues[] = $value;
				}
			}else{
				$keyStack[] = $key;
				$result = array_merge($result, $this->flatten_undefined($value, $keyStack));
				array_pop($keyStack);
			}
		}

		if(!empty($simpleValues)){
			if(empty($keyStack)){
				$result = array_merge($result, $simpleValues);
			}else{
				$result[] = implode(".", $keyStack) . '=' . implode(",", $simpleValues);
			}
		}
		return $result;
	}
}